from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from django.utils.decorators import method_decorator
from django.views.generic import View

from doi_site.settings import DATACITE_TEST_URL, DATACITE_URL

class Mint(View):

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(Mint, self).dispatch(*args, **kwargs)

    # pylint: disable=no-self-use
    def get(self, request):
        context = {'is_testing' : _is_test_url()}
        return render(request, 'datasets/mint.html', context)


def _is_test_url():
    if DATACITE_URL == DATACITE_TEST_URL:
        return True
    return False



""" Customisation for the admin interface. """

from django.contrib import admin
from django.contrib.auth.admin import GroupAdmin
from django.contrib.auth.models import Group

from .models import GroupProfile


# Register your models here.
class GroupProfileInline(admin.StackedInline):
    """
    The in-line display of the Group Profile.

    """
    model = GroupProfile
    can_delete = False
    verbose_name = 'Profile'
    verbose_name_plural = 'Profiles'
    list_display = ('doi_suffix')


# pylint: disable=too-many-public-methods
class MyGroupAdmin(GroupAdmin):
    """
    Customise GroupAdmin.

    """
    inlines = [GroupProfileInline]
    list_display = ('name', 'get_doi')

    # pylint: disable=no-self-use
    def get_doi(self, obj):
        """
        Get the DOI suffix.

        """
        doi_suffix = ''
        try:
            doi_suffix = obj.groupprofile.doi_suffix
        except AttributeError:
            pass
        return doi_suffix

    get_doi.short_description = 'doi suffix'


# Re-register GroupAdmin
admin.site.unregister(Group)
admin.site.register(Group, MyGroupAdmin)



'''
This module is used to make HTTP POST calls.
'''

import base64
import logging
import socket
from ssl import SSLError
import urllib2
from urlparse import urljoin

from django.http import HttpResponse

from doi_site.settings import DATACITE_URL, DOI_PREFIX, DATACITE_USER_NAME, \
    DATACITE_PASSWORD, TIME_OUT
from mds.http.helper import get_doi_from_request, get_opener, get_response, \
    is_authorized
import xml.etree.ElementTree as ET


LOGGING = logging.getLogger(__name__)


def post_doi(request):
    """
    Post the DOI.

    Args:
        request (HTTPRequest): The HTTP request

    Return:
        a HTTPResponse

    """
    LOGGING.info('Post doi')
    try:
        _doi = _get_doi_from_text_body(request.body)
    except IndexError:
        return get_response("Bad Request - request body must be exactly two " \
                            "lines: DOI and URL", 400)
    LOGGING.debug('Post doi, doi: %s', _doi)

    try:
        doi_suffix = _doi.split(DOI_PREFIX, 1)[1]
    except IndexError:
        return get_response("Bad Request - wrong prefix, doi should start " \
                            "with %s" % DOI_PREFIX, 400)

    if not is_authorized(request, doi_suffix):
        return get_response("Unauthorized - insufficient privileges", 403)

    url = urljoin(DATACITE_URL, request.get_full_path())
    return _post(url, request.body, _get_content_type_header(request))


def post_media(request):
    """
    Post the media.

    Args:
        request (HTTPRequest): The HTTP request

    Return:
        a HTTPResponse

    """
    LOGGING.info('Post media')
    _doi = get_doi_from_request(request, 'media')
    if _doi == None:
        return get_response("Bad Request - doi not found in URL", 400)
    LOGGING.debug('Post media, doi: %s', _doi)
    try:
        doi_suffix = _doi.split(DOI_PREFIX, 1)[1]
    except IndexError:
        return get_response("Bad Request - wrong prefix, doi should start " \
                            "with %s" % DOI_PREFIX, 400)

    if not is_authorized(request, doi_suffix):
        return get_response("Unauthorized - insufficient privileges", 403)

    url = urljoin(DATACITE_URL, request.get_full_path())
    return _post(url, request.body, _get_content_type_header(request))


def post_metadata(request):
    """
    Post the metadata.

    Args:
        request (HTTPRequest): The HTTP request

    Return:
        a HTTPResponse

    """
    LOGGING.info('Post metadata')
    try:
        _doi = _get_doi_from_xml_body(request.body)
    except ET.ParseError as ex:
        LOGGING.info('Error parsing xml from users request: %s', ex)
        return get_response("Bad Request - error parsing xml: %s" % ex, 400)
    if _doi == None:
        return get_response("Bad Request - doi not found in XML", 400)
    LOGGING.debug('Post metadata, doi: %s', _doi)
    try:
        doi_suffix = _doi.split(DOI_PREFIX, 1)[1]
    except IndexError:
        return get_response("Bad Request - wrong prefix, doi should start " \
                            "with %s" % DOI_PREFIX, 400)

    if not is_authorized(request, doi_suffix):
        return get_response("Unauthorized - insufficient privileges", 403)

    url = urljoin(DATACITE_URL, request.get_full_path())
    return _post(url, request.body, _get_content_type_header(request))


def _post(url, body, headers):
    """
    Send a post request to DataCite.

    Args:
        url (str): The URL to call
        body (str): The data
        headers (dict): A dictionary of headers to use

    Return:
        a HTTPResponse

    """
    _set_timeout()
    opener = get_opener()
    auth_string = (base64.encodestring(DATACITE_USER_NAME + ':'
                                       + DATACITE_PASSWORD)).rstrip()
    headers.update({'Authorization':'Basic ' + auth_string})

    # If the request body is a string, urllib2 attempts to concatenate the url,
    # body and headers. If the url is not UTF-8, the request body can get
    # converted unicode. This has resulted in issues where there are characters
    # with diacritic marks in the request body. To avoid these issues the url is
    # UTF-8 encoded.
    url_encode = url.encode('utf-8')

    req = urllib2.Request(url_encode, data=body, headers=headers)
    try:
        response = opener.open(req)
    except (urllib2.HTTPError) as ex:
        msg = ex.readlines()
        LOGGING.warn('HTTPError error getting %s. %s', url, msg)
        return get_response(msg, ex.code)
    except (socket.timeout, urllib2.URLError) as ex:
        LOGGING.warn('Timeout or URLError error getting %s. %s', url, ex.reason)
        return get_response(ex.reason, 500)
    except (SSLError) as ex:
        LOGGING.warn('SSLError error getting %s. %s', url, ex)
        return get_response(ex, 500)
    except UnicodeDecodeError as ex:
        LOGGING.info('UnicodeDecodeError error getting %s. %s', url, ex)
        return get_response(ex, 500)
    finally:
        _close(opener)
    if response.headers.has_key('Content-Type'):
        ret_response = HttpResponse(content_type=
                                    response.headers.get('Content-Type'))
    else:
        ret_response = HttpResponse()
    ret_response.status_code = response.code
    ret_response.reason_phrase = response.msg
    # pylint: disable=maybe-no-member
    ret_response.writelines(response.readlines())
    if response.headers.has_key('location'):
        ret_response.setdefault('Location', response.headers.get('location'))
    return ret_response


def _set_timeout():
    """
    Set the time out used by urllib.

    """
    socket.setdefaulttimeout(TIME_OUT)


def _close(opener):
    """
    Close the URL opener.

    """
    try:
        opener.close()
    # pylint: disable=bare-except
    except:
        pass


def _get_doi_from_text_body(body):
    """
    Get the DOI from the body of a request.

    Args:
        body (str): The data

    Return:
        a str containing the DOI

    """
    bits = body.split('url=', 1)
    if bits[0] != '':
        _doi = (bits[0].split('doi=')[1]).strip()
    else:
        bits = bits[1].split('doi=')
        _doi = bits[1].strip()

    return _doi


def _get_doi_from_xml_body(body):
    """
    Get the DOI from the body of a request where the DOI is embedded in the
    metadata XML.

    Args:
        body (str): The data

    Return:
        a str containing the DOI

    Throws:
        ParseError

    """
    root = ET.fromstring(body)
    for child in root.getchildren():
        if child.attrib == {'identifierType': 'DOI'}:
            return child.text
    return None


def _get_content_type_header(request):
    """
    Get the content type from the request. Return an empty dict if it is not set

    Args:
        request (HTTPRequest): The HTTP request

    Return:
        a dict containing the content type

    """
    try:
        return {'Content-Type':request.META['CONTENT_TYPE']}
    except KeyError:
        return {}



""" Models. """

from django.conf import settings
from django.db import models


# Create your models here.
class GroupProfile(models.Model):
    """
    Extension to Group model.
    """
    group = models.OneToOneField('auth.Group', unique=True)

    # The segment of the URI after the site DOI URI that defines this domain
    # within site DOI name space
    doi_suffix = models.CharField('DOI suffix - doi:' + settings.DOI_PREFIX
                                  + '/', max_length=100)

    def __unicode__(self):  # __unicode__ on Python 2
        return self.doi_suffix



""" This module provides the views for the web pages. """

# pylint: disable=no-self-use

from urlparse import urljoin

from django.contrib.auth.views import login as auth_login
from django.contrib.auth.views import logout as auth_logout
from django.shortcuts import render
from django.views.generic import View
from django.views.generic.list import ListView

from doi_site import settings
from doi_site.settings import DATACITE_HANDLER, DATACITE_TEST_URL, \
    DATACITE_URL, DOI_PREFIX, ORGANISATION_NAME, ORGANISATION_DOI_EMAIL
from mds.http.get import get as _get
from mds.http.helper import get_accept_header
from mds.models import GroupProfile


class DoiList(View):
    """
    Display all the DOIs for this organisation.

    """

    def dispatch(self, *args, **kwargs):
        return super(DoiList, self).dispatch(*args, **kwargs)

    def get(self, request):
        """
        Get the list of DOIs.

        """
        url = urljoin(DATACITE_URL, "doi")
        context = {'is_testing' : _is_test_url()}
        response = _get(request.method, url, get_accept_header(request))
        if response.status_code != 200:
            context['message'] = response.content
            return render(request, 'doi_site/error.html', context)
        context['handler'] = DATACITE_HANDLER
        dois = []
        for line in response:
            if line != '':
                dois.append(line.strip())
        context['dois'] = dois
        return render(request, 'doi_site/dois.html', context)


class HomeView(View):
    """
    Display the home page.

    """

    def get(self, request):
        """
        Get the home page.

        """
        context = {'organisation_name' : ORGANISATION_NAME}
        context['organisation_email'] = ORGANISATION_DOI_EMAIL
        context['is_testing'] = _is_test_url()
        return render(request, 'doi_site/index.html', context)


class Notes(View):
    """
    Display the notes page.

    """

    def dispatch(self, *args, **kwargs):
        return super(Notes, self).dispatch(*args, **kwargs)

    def get(self, request):
        """
        Get the notes page.

        """
        context = {'organisation_name' : ORGANISATION_NAME}
        context['is_testing'] = _is_test_url()
        context['roles'] = getattr(settings, 'ROLES_URL', '')
        context['notes'] = getattr(settings, 'NOTES_URL', '')
        return render(request, 'doi_site/notes.html', context)


# pylint: disable=too-many-ancestors
class Domains(ListView):
    """
    Display the list of DOI domains.

    """

    model = GroupProfile
    context_object_name = 'group_profiles'
    template_name = 'doi_site/domains.html'

    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super(Domains, self).get_context_data(**kwargs)
        context['doi_prefix'] = DOI_PREFIX + '/'
        context['is_testing'] = _is_test_url()
        return context


def login(request):
    """
    Display the login page.

    """
    context = {'is_testing' : _is_test_url()}
    context['organisation_name'] = ORGANISATION_NAME
    return auth_login(request, extra_context=context)


def logout(request):
    """
    Respond to a logout request.

    """
    context = {'is_testing' : _is_test_url()}
    return auth_logout(request, extra_context=context)


def _is_test_url():
    """
    Are we using the DataCite test service?

    Return:
        a boolean, True if we are using the DataCite test service

    """
    if DATACITE_URL == DATACITE_TEST_URL:
        return True
    return False




'''
This module is used to make HTTP DELETE calls.
'''

import base64
import logging
import socket
from ssl import SSLError
import urllib2
from urlparse import urljoin

from django.http import HttpResponse

from doi_site.settings import DATACITE_URL, DOI_PREFIX, DATACITE_USER_NAME, \
    DATACITE_PASSWORD, TIME_OUT
from mds.http.helper import get_doi_from_request, get_opener, get_response, \
    is_authorized


LOGGING = logging.getLogger(__name__)


def delete_metadata(request):
    """
    Delete the metadata for the DOI.

    Args:
        request (HTTPRequest): The HTTP request

    Return:
        a HTTPResponse

    """
    _doi = get_doi_from_request(request, 'metadata')
    LOGGING.info('Delete metadata doi: %s', _doi)
    url = urljoin(DATACITE_URL, request.get_full_path())
    try:
        doi_suffix = _doi.split(DOI_PREFIX, 1)[1]
    except IndexError:
        return get_response("Bad Request - wrong prefix, doi should start " \
                            "with %s" % DOI_PREFIX, 400)

    if not is_authorized(request, doi_suffix):
        return get_response("Unauthorized - insufficient privileges", 403)
    return _delete(url)


def _delete(url):
    """
    Send a delete request to DataCite.

    Args:
        url (str): The URL to call

    Return:
        a HTTPResponse

    """
    _set_timeout()
    opener = get_opener()
    auth_string = (base64.encodestring(DATACITE_USER_NAME + ':'
                                       + DATACITE_PASSWORD)).rstrip()
    headers = {'Authorization':'Basic ' + auth_string}
    req = urllib2.Request(url, None, headers)
    req.get_method = lambda: 'DELETE'
    try:
        response = opener.open(req)
    except (urllib2.HTTPError) as ex:
        msg = ex.readlines()
        LOGGING.warn('HTTPError error getting %s. %s', url, msg)
        return get_response(msg, ex.code)
    except (socket.timeout, urllib2.URLError) as ex:
        LOGGING.warn('Timeout or URLError error getting %s. %s', url, ex.reason)
        return get_response(ex.reason, 500)
    except (SSLError) as ex:
        LOGGING.warn('SSLError error getting %s. %s', url, ex)
        return get_response(ex, 500)
    finally:
        _close(opener)
    if response.headers.has_key('Content-Type'):
        ret_response = HttpResponse(content_type=
                                    response.headers.get('Content-Type'))
    else:
        ret_response = HttpResponse()
    ret_response.status_code = response.code
    ret_response.reason_phrase = response.msg
    # pylint: disable=maybe-no-member
    ret_response.writelines(response.readlines())
    return ret_response


def _set_timeout():
    """
    Set the time out used by urllib.

    """
    socket.setdefaulttimeout(TIME_OUT)


def _close(opener):
    """
    Close the URL opener.

    """
    try:
        opener.close()
    # pylint: disable=bare-except
    except:
        pass



""" This module provides the views for the ReST API. """

# pylint: disable=no-self-use

from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from doi_site.settings import DATACITE_URL
from mds.basic_auth import logged_in_or_basicauth
from mds.http.delete import delete_metadata
from mds.http.get import get as _get
from mds.http.helper import get_accept_header
from mds.http.post import post_doi, post_media, post_metadata
from urlparse import urljoin


class MetadataView(View):
    """
    Handle delete, head and get requests for metadata.

    """
    @method_decorator(csrf_exempt)
    @method_decorator(logged_in_or_basicauth())
    def dispatch(self, *args, **kwargs):
        return super(MetadataView, self).dispatch(*args, **kwargs)

    def delete(self, request):
        """
        This request marks a dataset as 'inactive'. To activate it again, POST
        new metadata or set the isActive-flag in the user interface.

        """
        return delete_metadata(request)

    def get(self, request):
        """
        This request returns the most recent version of metadata associated with
        a given DOI.

        """
        url = urljoin(DATACITE_URL, request.get_full_path())
        return _get(request.method, url, get_accept_header(request))

    def head(self, request):
        """
        Request the most recent version of metadata associated with a given DOI.

        """
        url = urljoin(DATACITE_URL, request.get_full_path())
        return _get(request.method, url, get_accept_header(request))


class MetadataPost(View):
    """
    Handle post requests for metadata.

    """
    @method_decorator(csrf_exempt)
    @method_decorator(logged_in_or_basicauth())
    def dispatch(self, *args, **kwargs):
        return super(MetadataPost, self).dispatch(*args, **kwargs)

    def post(self, request):
        """
        This request stores new version of metadata. The request body must
        contain valid XML.

        """
        return post_metadata(request)


class DoiView(View):
    """
    Handle post requests for DOIs and get and head requests for all DOIs.

    """
    @method_decorator(csrf_exempt)
    @method_decorator(logged_in_or_basicauth())
    def dispatch(self, *args, **kwargs):
        return super(DoiView, self).dispatch(*args, **kwargs)

    def get(self, request):
        """
        This request returns a list of all DOIs for the requesting datacentre.
        There is no guaranteed order.

        """
        url = urljoin(DATACITE_URL, request.get_full_path())
        return _get(request.method, url, get_accept_header(request))

    def head(self, request):
        """
        Request a list of all DOIs for the requesting datacentre.

        """
        url = urljoin(DATACITE_URL, request.get_full_path())
        return _get(request.method, url, get_accept_header(request))

    def post(self, request):
        """
        POST will mint new DOI if specified DOI doesn't exist. This method will
        attempt to update URL if you specify existing DOI. Standard domains and
        quota restrictions check will be performed. A Datacentre's doiQuotaUsed
        will be increased by 1. A new record in Datasets will be created.

        """
        return post_doi(request)


class DoiDetail(View):
    """
    Handle get and head requests for an individual DOIs.

    """

    @method_decorator(logged_in_or_basicauth())
    def dispatch(self, *args, **kwargs):
        return super(DoiDetail, self).dispatch(*args, **kwargs)

    def get(self, request):
        """
        This request returns an URL associated with a given DOI.

        """
        url = urljoin(DATACITE_URL, request.get_full_path())
        return _get(request.method, url, get_accept_header(request))

    def head(self, request):
        """
        Request an URL associated with a given DOI.

        """
        url = urljoin(DATACITE_URL, request.get_full_path())
        return _get(request.method, url, get_accept_header(request))


class MediaView(View):
    """
    Handle head, get and post requests for media.

    """

    @method_decorator(csrf_exempt)
    @method_decorator(logged_in_or_basicauth())
    def dispatch(self, *args, **kwargs):
        return super(MediaView, self).dispatch(*args, **kwargs)

    def get(self, request):
        """
        This request returns list of pairs of media type and URLs associated
        with a given DOI.

        """
        url = urljoin(DATACITE_URL, request.get_full_path())
        return _get(request.method, url, get_accept_header(request))

    def head(self, request):
        """
        Request list of pairs of media type and URLs associated with a given
        DOI.

        """
        url = urljoin(DATACITE_URL, request.get_full_path())
        return _get(request.method, url, get_accept_header(request))

    def post(self, request):
        """
        POST will add/update media type/urls pairs to a DOI. Standard domain
        restrictions check will be performed.

        """
        return post_media(request)




import os

from setuptools import setup


README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='doi_site',
    version='0.1.1',
    packages=['doi_site', 'mds', 'datasets'],
    include_package_data=True,
    license='BSD License',
    description='A simple Django app to wrap datacite calls.',
    long_description=README,
    url='http://stfc.ac.uk/',
    author='Antony Wilson',
    author_email='antony.wilson@stfc.ac.uk',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],


    # Adds dependencies
    install_requires=[
        'Django==1.8',
        'django-auth-ldap',
        'httplib2',
        'python-ldap',
    ],
)



"""
This class was taken from djangosnippits.

https://djangosnippets.org/snippets/243/

Author Scanner, https://djangosnippets.org/users/Scanner/

"""
import base64

from django.contrib.auth import authenticate, login
from django.http import HttpResponse


def view_or_basicauth(view, request, test_func, realm="", *args, **kwargs):
    """
    This is a helper function used by both 'logged_in_or_basicauth' and
    'has_perm_or_basicauth' that does the nitty of determining if they
    are already logged in or if they have provided proper http-authorization
    and returning the view if all goes well, otherwise responding with a 401.
    """
    if test_func(request.user):
        # Already logged in, just return the view.
        return view(request, *args, **kwargs)

    # They are not logged in. See if they provided login credentials
    if 'HTTP_AUTHORIZATION' in request.META:
        auth = request.META['HTTP_AUTHORIZATION'].split()
        if len(auth) == 2:
            # NOTE: We are only support basic authentication for now.
            #
            if auth[0].lower() == "basic":
                uname, passwd = base64.b64decode(auth[1]).split(':', 1)
                user = authenticate(username=uname, password=passwd)
                if user is not None:
                    if user.is_active:
                        login(request, user)
                        request.user = user
                        if test_func(request.user):
                            return view(request, *args, **kwargs)

    # Either they did not provide an authorization header or
    # something in the authorization attempt failed. Send a 401
    # back to them to ask them to authenticate.
    #
    response = HttpResponse()
    response.status_code = 401
    response['WWW-Authenticate'] = 'Basic realm="%s"' % realm
    return response


def logged_in_or_basicauth(realm=""):
    """
    A simple decorator that requires a user to be logged in. If they are not
    logged in the request is examined for a 'authorization' header.

    If the header is present it is tested for basic authentication and
    the user is logged in with the provided credentials.

    If the header is not present a http 401 is sent back to the
    requestor to provide credentials.

    The purpose of this is that in several django projects I have needed
    several specific views that need to support basic authentication, yet the
    web site as a whole used django's provided authentication.

    The uses for this are for urls that are access programmatically such as
    by rss feed readers, yet the view requires a user to be logged in. Many rss
    readers support supplying the authentication credentials via http basic
    auth (and they do NOT support a redirect to a form where they post a
    username/password.)

    Use is simple:

    @logged_in_or_basicauth
    def your_view:
        ...

    You can provide the name of the realm to ask for authentication within.
    """
    def view_decorator(func):
        def wrapper(request, *args, **kwargs):
            return view_or_basicauth(func, request,
                                     lambda u: u.is_authenticated(),
                                     realm, *args, **kwargs)
        return wrapper
    return view_decorator



"""
Django settings for doi_site project.

Generated by 'django-admin startproject' using Django 1.8.

For more information on this file, see
https://docs.djangoproject.com/en/1.8/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.8/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

# A list of strings representing the host/domain names that this Django site can
# serve
ALLOWED_HOSTS = []

# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'datasets',
    'mds',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.security.SecurityMiddleware',
)

ROOT_URLCONF = 'doi_site.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'doi_site', 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'doi_site.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.8/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}


# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/

STATIC_URL = '/static/'

STATIC_ROOT = '/var/www/html/doi/'

STATICFILES_DIRS = (
    os.path.join(BASE_DIR, "doi_site", "static"),
)

# Organisation's DataCite prefix in the form nn.nnnn
DOI_PREFIX = ''

# Datacite test URL
DATACITE_URL = 'https://mds.test.datacite.org'

# Used to compare against DATACITE_URL to check if the test message should be
# displayed
DATACITE_TEST_URL = 'https://mds.test.datacite.org'

# Datacite test handler
DATACITE_HANDLER = 'http://test.datacite.org/handle/'

# The connection timeout in seconds
TIME_OUT = 10

# Populate the Django user from the LDAP directory.
AUTH_LDAP_USER_ATTR_MAP = {
    "first_name": "givenName",
    "last_name": "sn",
    "email": "mail"
}

# This is the default, but I like to be explicit.
AUTH_LDAP_ALWAYS_UPDATE_USER = True

# Keep ModelBackend around for per-user permissions and maybe a local
# superuser.
AUTHENTICATION_BACKENDS = (
    'django_auth_ldap.backend.LDAPBackend',
    'django.contrib.auth.backends.ModelBackend',
)

LOGIN_REDIRECT_URL = 'index'

# The name of your organisation to display on the home page
ORGANISATION_NAME = 'My Organisation'

# The email address to display on the home page
ORGANISATION_DOI_EMAIL = 'My eMail'


try:
    #pylint: disable=wildcard-import, unused-wildcard-import
    from doi_site.local_settings import *
except ImportError:
    pass



""" The URL mappings. """

from django.conf.urls import include, url
from django.contrib import admin

from datasets.views import Mint
from doi_site.views import DoiList, Domains, HomeView, Notes
from mds.views import DoiView, DoiDetail, MetadataPost, MetadataView, MediaView


# pylint: disable=invalid-name
urlpatterns = [
    url(r'^$', HomeView.as_view(), name='home'),
    url(r'^index$', HomeView.as_view(), name='index'),

    url(r'^admin/', include(admin.site.urls)),

    # web
    url(r'^doilist$', DoiList.as_view(), name='doi_list'),
    url(r'^domains$', Domains.as_view(), name='domains'),
    url(r'^mint$', Mint.as_view(), name='mint'),
    url(r'^notes$', Notes.as_view(), name='notes'),

    # MDS
    url(r'^doi$', DoiView.as_view(), name='doi_view'),
    url(r'^doi/', DoiDetail.as_view(), name='doi_detail'),
    url(r'^metadata$', MetadataPost.as_view(), name='metadata_post'),
    url(r'^metadata/', MetadataView.as_view(), name='metadata_view'),
    url(r'^media/', MediaView.as_view(), name='media_view'),

    url(r'^accounts/login/$', 'doi_site.views.login', name='login'),
    url(r'^logout/$', 'doi_site.views.logout', name='logout'),

    url(r'', HomeView.as_view()),
]



'''
This module is used to make HTTP HEAD and GET calls.
'''

import base64
import logging
import socket
from ssl import SSLError
import urllib2

from django.http import HttpResponse

from doi_site.settings import DATACITE_USER_NAME, DATACITE_PASSWORD, TIME_OUT
from mds.http.helper import get_response, get_opener


LOGGING = logging.getLogger(__name__)


def get(request_method, url, headers):
    """
    Send a get or head request to DataCite.

    Args:
        request_method (str): This should be 'GET' or 'HEAD'
        url (str): The URL to call
        headers (dict): A dictionary of headers to use

    Return:
        a HTTPResponse

    """
    LOGGING.info('get(%s,%s,%s)', request_method, url, headers)
    _set_timeout()
    opener = get_opener()
    auth_string = (base64.encodestring(DATACITE_USER_NAME + ':'
                                       + DATACITE_PASSWORD)).rstrip()
    headers.update({'Authorization':'Basic ' + auth_string})
    req = urllib2.Request(url, None, headers)
    if request_method == "HEAD":
        req.get_method = lambda: 'HEAD'
    try:
        response = opener.open(req)
    except (urllib2.HTTPError) as ex:
        msg = ex.readlines()
        LOGGING.warn('HTTPError error getting %s. %s', url, msg)
        return get_response(msg, ex.code)
    except (socket.timeout, urllib2.URLError) as ex:
        LOGGING.warn('Timeout or URLError error getting %s. %s', url, ex.reason)
        return get_response(ex.reason, 500)
    except (SSLError) as ex:
        LOGGING.warn('SSLError error getting %s. %s', url, ex)
        return get_response(ex, 500)
    finally:
        _close(opener)
    if response.headers.has_key('Content-Type'):
        ret_response = HttpResponse(content_type=
                                    response.headers.get('Content-Type'))
    else:
        ret_response = HttpResponse()
    ret_response.status_code = response.code
    ret_response.reason_phrase = response.msg
    # pylint: disable=maybe-no-member
    ret_response.writelines(response.readlines())
    return ret_response


def _set_timeout():
    """
    Set the time out used by urllib.

    """
    socket.setdefaulttimeout(TIME_OUT)


def _close(opener):
    """
    Close the URL opener.

    """
    try:
        opener.close()
    # pylint: disable=bare-except
    except:
        pass



'''
Helper methods/ common functionality.
'''

import urllib2

from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse

from doi_site import settings


def is_authorized(request, doi_suffix):
    """
    Check if the user is authorised to use the suffix.

    Args:
        request (HTTPRequest): The HTTP request
        doi_suffix (str): The DOI suffix

    Return:
        a boolean, True if the user is authorised

    """
    if doi_suffix.startswith('/'):
        doi_suffix = doi_suffix.split('/', 1)[1]
    authorized_dois = []
    groups = request.user.groups.iterator()
    for group in groups:
        try:
            authorized_dois.append(group.groupprofile.doi_suffix)
        except ObjectDoesNotExist:
            pass
    for auth_doi in authorized_dois:
        if doi_suffix.startswith(auth_doi):
            return True

    return False


def get_accept_header(request):
    """
    Extract the accept header from the request.

    Args:
        request (HTTPRequest): The HTTP request

    Return:
        a dict, if present key = 'Accept'

    """
    try:
        return {'Accept':request.META['HTTP_ACCEPT']}
    except KeyError:
        return {}


def get_doi_from_request(request, method):
    """
    Extract the doi from the request.

    Args:
        request (HTTPRequest): The HTTP request

    Return:
        a str, the DOI

    """
    full_path = request.get_full_path()
    bits = full_path.split(method + '/')
    try:
        return bits[1]
    except IndexError:
        return None


def get_response(message, code):
    """
    Create a response.

    Args:
        message (str): The message to include in the response
        code (str): The return code to include in the response

    Return:
        a dict, if present key = 'Accept'

    """
    response = HttpResponse(status=code)
    # pylint: disable=no-member
    response.writelines(message)
    return response


def get_opener():
    """
    Get a http opener.
    A check is made to see if a proxy should be used.
    
    Return:
        an opener object

    """
    if _is_use_proxy():
        proxy_handler = urllib2.ProxyHandler({'https': getattr(settings, 
            'HTTP_PROXY_HOST') + ":" + getattr(settings, 'HTTP_PROXY_PORT')})
        return urllib2.build_opener(proxy_handler)
    else:
        return urllib2.build_opener()


def _is_use_proxy():
    """
    Test to see if we should be using a http proxy.

    Return:
        boolean, True to use proxy

    """
    if (getattr(settings, 'HTTP_PROXY_HOST', False) and
        getattr(settings, 'HTTP_PROXY_PORT', False)):
        return True
    return False
